/***************************************************************************
            dchublistitems.h  -  Items used by DCHubListManager
                             -------------------
    begin                : Wed Jul 3 2008
    copyright            : (C) 2008 by Edward Sheldrake
    email                : ejs1920@yahoo.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCHUBLISTITEMS_H
#define DCHUBLISTITEMS_H

/**
  * @author Edward Sheldrake
  *
  * Modified QTreeWidgetItem that has an int attached to it.
  * This file is not as necessary as the QT3 version which has much more.
  */

#include <QTreeWidget>

class DCBookmarkHubItem : public QTreeWidgetItem {

public:
	/** constructor */
	DCBookmarkHubItem( QTreeWidget * parent ) : QTreeWidgetItem( parent ) { };
	/** constructor */
	DCBookmarkHubItem( QTreeWidgetItem * parent ) : QTreeWidgetItem( parent ) { };
	/** destructor */
	virtual ~DCBookmarkHubItem() { };
	
	/**
	 * It's really the key of the item in CConfig's ordered bookmark map, which may or
	 * may not be the actual position of the item in the QTreeWidget.
	 */
	int position;
};

#endif // DCHUBLISTITEMS_H
