/***************************************************************************
                          dcpluginmanager.cpp  -  description
                             -------------------
    begin                : Don Dez 5 2002
    copyright            : (C) 2002 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dcpluginmanager.h"

#include <stdio.h>

#include <dclib/core/cplugin.h>

/** */
DCPluginManager * g_pPluginManager = 0;

/** */
DCPluginManager::DCPluginManager()
{
	g_pPluginManager = this;
}

/** */
DCPluginManager::~DCPluginManager()
{
	g_pPluginManager = NULL;
}

/** */
bool DCPluginManager::Init( CPluginObject * plugin )
{
	if ( plugin->m_ePluginStruct->m_eType != eptGUI )
	{
		printf("Plugin != GUI not for me ...\n");
		return CPluginManager::Init(plugin);
	}

	return true;
}
