/***************************************************************************
                          cdialogpicturemap.h  -  description
                             -------------------
    begin                : Sat Apr 26 2003
    copyright            : (C) 2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDIALOGPICTUREMAP_H
#define CDIALOGPICTUREMAP_H

#include <qdialog.h>
//Added by qt3to4:
#include <QEvent>
#include <QGridLayout>
#include <QPixmap>
#include <QLabel>

/**
  *@author Mathias Küster
  */

class CDialogPictureMap : public QDialog {
Q_OBJECT

public:
	/** construtor */
	CDialogPictureMap( QWidget * parent = 0, const char * name = 0, bool modal = FALSE, Qt::WFlags f = 0 );
	/** destructor */
	virtual ~CDialogPictureMap();

	/** */
	void SetPixmap( QPixmap & pixmap );
	/** */
	void GetXY( int & x, int & y );

protected:
	/** */
	bool eventFilter( QObject * object, QEvent * event );

private:
	/** */
	QLabel * m_pLabel;
	/** */
	QGridLayout * m_pGridLayout;
	/** */
	int m_nX,m_nY;
};

#endif
