/***************************************************************************
                          dchubprofile.h  -  description
                             -------------------
    begin                : Fre Sep 20 2002
    copyright            : (C) 2002 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCHUBPROFILE_H
#define DCHUBPROFILE_H

#include <qwidget.h>
#include "DCDialogHubProfile.h"

/**
  *@author Mathias Küster
  */

class DCHubProfile : public QDialog, private Ui::DCDialogHubProfile  {
Q_OBJECT

public: 
	/** construtor */
	DCHubProfile( QWidget * parent = 0 );
	/** destructor */
	virtual ~DCHubProfile();

	/** */
	void InitDocument();

private:
	/** */
	void SaveProfile( QString profilename );

	/** */
	QString m_sOldName;

private slots:
	/** */
	void slotClickedOK();
	/** */
	void slotProfileActiveted( const QString & );
	/** Called when the Delete button is pressed to delete a profile */
	void slotProfileDelete();
};

#endif
