/***************************************************************************
                          dceditserver.h  -  description
                             -------------------
    begin                : Fre Aug 29 2003
    copyright            : (C) 2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCEDITSERVER_H
#define DCEDITSERVER_H

#include <qwidget.h>
#include "DCDialogEditServer.h"

/**
  *@author Mathias Küster
  */

class DCConfigHubProfile;

class DCEditServer : public QDialog, public Ui::DCDialogEditServer  {
Q_OBJECT

public: 
	/** construtor */
	DCEditServer( QWidget * parent = 0 );
	/** destructor */
	virtual ~DCEditServer();

	/** */
	void Reset();
	/** */
	void SetProfile( DCConfigHubProfile * profile );
	/** */
	void GetProfile( DCConfigHubProfile * profile );

public slots:
	/** checks name and host are not empty */
	virtual void accept();

private slots:
	/** */
	void slotProfileButtonClicked();
	/** */
	void slotCheckBoxProfileToggled(bool);
};

#endif
