/***************************************************************************
                          dchubfilter.h  -  description
                             -------------------
    begin                : Mit Okt 30 2002
    copyright            : (C) 2002 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCHUBFILTER_H
#define DCHUBFILTER_H

#include <qwidget.h>
#include "DCDialogHubFilter.h"

class DC_HubFilterObject;

/**
  *@author Mathias Küster
  */

class DCHubFilter : public QDialog, private Ui::DCDialogHubFilter  {
Q_OBJECT

public: 
	/** construtor */
	DCHubFilter( QWidget * parent = 0 );
	/** destructor */
	virtual ~DCHubFilter();

	/** */
	void Edit( DC_HubFilterObject * HubFilterObject );
	/** */
	void Get( DC_HubFilterObject * HubFilterObject );
};

#endif
