/***************************************************************************
                          dcapplicationevents.cpp  -  description
                             -------------------
    begin                : Don Aug 21 2003
    copyright            : (C) 2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dcapplicationevents.h"

/** */
DCApplicationEvents::DCApplicationEvents()
{
	m_nNoUserInputTimer = 0;
}

/** */
DCApplicationEvents::~DCApplicationEvents()
{
}

/** */
bool DCApplicationEvents::eventFilter(QObject* object, QEvent* event)
{
	// away handling
	if ( (event->type() == QEvent::MouseButtonPress) ||
	     (event->type() == QEvent::MouseButtonRelease) ||
	     (event->type() == QEvent::MouseButtonDblClick) ||
	     (event->type() == QEvent::MouseMove) ||
	     (event->type() == QEvent::KeyPress) ||
	     (event->type() == QEvent::KeyRelease) ||
	     (event->type() == QEvent::Wheel) )
	{
		m_nNoUserInputTimer = 0;
	}

	return QObject::eventFilter( object, event );    // standard event processing
}
