/***************************************************************************
                          cdialogmessage.h  -  description
                             -------------------
    begin                : Don Sep 18 2003
    copyright            : (C) 2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CDIALOGMESSAGE_H
#define CDIALOGMESSAGE_H

#include <qwidget.h>
#include <qmessagebox.h>

#include "DCDialogMessage.h"

/**
  *@author Mathias Küster
  */

class CDialogMessage : public QDialog, public Ui::DCDialogMessage  {
Q_OBJECT

public: 
	/** construtor */
	CDialogMessage( QWidget *parent,
			enum QMessageBox::Icon icon,
			QString caption,
			QString text,
			const QString & button0Text = QString::null,
			const QString & button1Text = QString::null,
			const QString & button2Text = QString::null );
	/** destructor */
	virtual ~CDialogMessage();

	/** */
	bool GetCheckBoxStatus();

private:
	/** construtor */
	CDialogMessage() {};

private slots:
	/** */
	void slotPushButton0Clicked();
	/** */
	void slotPushButton1Clicked();
	/** */
	void slotPushButton2Clicked();
};

#endif
