/***************************************************************************
                          dcapplicationevents.h  -  description
                             -------------------
    begin                : Don Aug 21 2003
    copyright            : (C) 2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCAPPLICATIONEVENTS_H
#define DCAPPLICATIONEVENTS_H

#include <qobject.h>
#include <qevent.h>

/**
  *@author Mathias Küster
  */

class DCApplicationEvents : public QObject
{
Q_OBJECT

public:
	/** construtor */
	DCApplicationEvents();
	/** destructor */
	virtual ~DCApplicationEvents();

	/** user input event timer */
	unsigned int m_nNoUserInputTimer;

protected:
	/** */
	virtual bool eventFilter( QObject* object, QEvent* event );
};

#endif
