/***************************************************************************
                          dcfiletransferinfo.h  -  description
                             -------------------
    begin                : Sam Jul 13 2002
    copyright            : (C) 2002-2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCFILETRANSFERINFO_H
#define DCFILETRANSFERINFO_H

/**
  *@author Mathias Küster
  */

#include <qtimer.h>

#include <dclib/core/cstring.h>

#include "DCDialogFileTransferInfo.h"

class DCFileChunkObject;

class DCFileTransferInfo : public QWidget, private Ui::DCDialogFileTransferInfo {
Q_OBJECT

public: 
	/** construtor */
	DCFileTransferInfo( CString filename, QWidget * parent );
	/** destructor */
	virtual ~DCFileTransferInfo();

private:
	/** */
	void InitDocument();
	
	/** */
	QTimer m_Timer;
	/** */
	CString m_sFileName;

private slots:
	/** */
	void timerDone();
};

#endif
