/***************************************************************************
     search-result-columns.h  -  Column numbers for search results list
                             -------------------
    begin                : Sun Jan 18 2009
    copyright            : (C) 2009 by Edward Sheldrake
    email                : ejs1920@yahoo.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* They are needed in dcwidget.cpp */

#ifndef SEARCH_RESULT_COLUMNS
#define SEARCH_RESULT_COLUMNS

/* column numbers for the search results listview */
#define RESULTS_COLUMN_COUNT		0
#define RESULTS_COLUMN_FILE		1
#define RESULTS_COLUMN_SIZE		2
#define RESULTS_COLUMN_TTH		3
#define RESULTS_COLUMN_PATH		4
#define RESULTS_COLUMN_NICK		5
#define RESULTS_COLUMN_FREESLOTS	6
#define RESULTS_COLUMN_TOTALSLOTS	7
#define RESULTS_COLUMN_IP		8
#define RESULTS_COLUMN_HUB		9
#define RESULTS_COLUMN_HOST		10

#endif /* SEARCH_RESULT_COLUMNS */
